#include "node.h"
#include <iostream>
#include <string>
#include "transformnode.h"
#include <algorithm>

Node::Node(std::string name)
{
    this->name = name;
    this->type = "Node";
    children = std::vector<Node*>(0);
    parent = NULL;

}

Node::~Node(){
    for (unsigned int i = 0; i < children.size(); i++){
        delete children[i];
    }
}

std::string Node::getType(){
    return this->type;
}

std::string Node::getName(){
    return this->name;
}

Node* Node::getParent(){
    return this->parent;
}

void Node::setParent(Node* parent){
    this->parent = parent;
}

std::vector<Node*> Node::getChildren(){
    return this->children;
}

void Node::applySelf(){
    //empty for generic type
}

void Node::apply(){
    applySelf();
    for (unsigned int i = 0; i < children.size(); i++){
        glPushMatrix();
        children[i]->apply();
        glPopMatrix();
    }
}

void Node::addChild(Node *child){
    if (child == NULL){ return; }
//    std::cout << "Node \"" << name << "\" added child " << child->name << " , children now: " << children.size() << std::endl;
    children.push_back(child);
    child->setParent(this);
}

void Node::print(int indent){
    std::cout << "Node \"" << name << "\", Type: " << type << ", " << children.size() << " children" << std::endl;
    for (unsigned int i = 0; i < children.size(); i++){
        for (int j = 0; j < indent+1; j++){ std::cout << "\t"; } // indent children by one
        children[i]->print(indent+1);
    }

}

void Node::removeChild(Node* child)
{
    std::vector<Node*>::iterator found = std::find(children.begin(), children.end(), child);
    if (found != children.end())
    {
        children.erase(found);
    }
}

Node* Node::findChildNodeByName(const std::string &nodeName)
{
    if (this->name == nodeName)
        return this;
    else
        for (std::vector<Node*>::iterator it = children.begin(); it != children.end(); it++)
        {
            Node* found = (*it)->findChildNodeByName(nodeName);
            if (found)
                return found;
        }
    return NULL;
}
